<?php
/* -------------------------------------------------------------
Remote Execution Framework (REX) - Launcher Script

This file is part of the Remote Execution Framework (REX)
 
REX is (C) Copyright 2010 David Cutting
 
REX is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

REX is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
 
You should have received a copy of the GNU General Public License
along with REX.  If not, see www.gnu.org/licenses
 
For more information see http://rex.purplepixie.org/
-------------------------------------------------------------- */

require("rex.inc.php");

$CLI=false;
$HTTP=false;

if (php_sapi_name()=="cli") $CLI=true;
else $HTTP=true;


function YesNo($bool)
{
	if ($bool) return "Yes";
	else return "No";
}

function CLI_Usage()
{
	echo "Usage: php rex.php [options] path-to-rex-code [path-to-rex-code ...]\n";
	echo "See php rex.php --help for more information\n";
}

function CLI_Help()
{
	echo "REX - Remote Execution Framework for PHP\n\n";
	echo "Usage: php rex.php [options] rex-code [rex-code ...]\n\n";
	echo "Options:\n\n";
	echo " --help  Display this help and exit\n\n";
	echo " --quiet=X  Turn REX quiet (no exec errors) with X=1 or noisy with X=0\n";
	echo " default: 0 (not quiet)\n\n";
	echo " --mute=X  If muted (1) errors found during file loads etc are not shown\n";
	echo " default: 0 (not muted)\n\n";
	echo " --blocks=X  Size of blocks (bytes) to load from stream (default 4096)\n\n";
	echo " --base=X  Set REX codebase to this directory (can be quoted), if not\n";
	echo " specified base will be directory of first REX execution\n\n";
	echo " --default=X  Set REX default file (default is index.rex)\n\n";
	echo " --continue=X  REX will continue to next file if error is found (1) or not (0)\n";
	echo " default 0 (stop execution)\n\n";
	echo " --debug=X  Display debug output (1) or not (0), default 0\n\n";
	echo " --set VAR=VAL  Set a REX variable of VAR to VAL, both VAR and VAL can be quoted\n\n";
	echo " --env VAR=VAL  Set a REX Environment variable of VAR to VAL (can both be quoted)\n\n";
	echo "Filename(s):\n\n";
	echo "List of filename(s) to execute\n\n";
	echo "Examples:\n\n";
	echo "php rex.php http://server/rex-code/\n";
	echo "Will execute rex-code/index.rex on server\n\n";
	echo "php rex.php --mute=1 --continue=1 http://server/rex-code/mycode.rex also.rex\n";
	echo "Will execute rex-code/mycode.rex on server muting all errors and then move\n";
	echo "onto rex-code/also.rex on server regardless if mycode.rex generated an error\n\n";
	echo "For more information see: http://rex.purplepixie.org/\n\n";
}

function Strip_Quotes($string)
{
	if ($string[0]=="\"") $string=substr($string,1);
	if ($string[strlen($string)-1]=="\"") $string=substr($string,0,strlen($string)-1);
	return $string;
}

function Newline()
{
	global $HTTP,$CLI;
	if ($HTTP) return "<BR />\n";
	else return "\n";
}

// These variables are used by the launcher script not REX itself
$filenames = array();
$mute = false;
$continue = false;
$debug = false;


// Insntiate REX
$REX = new REX();


// Parse the options
if ($CLI) // CLI mode
{
	if ($argc<2)
	{
		CLI_Usage();
		exit();
	}
	
	for ($a=1; $a< $argc; $a++)
	{
		$opt = $argv[$a];
		if (substr($opt,0,2)=="--") // command line switch
		{
			$opt=substr($opt,2);
			$equalpos=strpos($opt,"=");
			if ($equalpos===false)
			{
				$val=0;
			}
			else
			{
				$val=Strip_Quotes(substr($opt,$equalpos+1));
				$opt=substr($opt,0,$equalpos);
			}
			
			echo "Parsing opt: [".$opt."] val: [".$val."]\n";
			
			switch($opt)
			{
				case "help":
					CLI_Help();
					exit();
					break;
				case "quiet":
					if ($val==1) $REX->Quiet=true;
					else $REX->Quiet=false;
					break;
				case "mute":
					if ($val==1) $mute=true;
					else $mute=false;
					break;
				case "blocks":
					if (is_numeric($val) && ($val>0)) $REX->Blocks=$val;
					break;
				case "base":
					$REX->Base=$val;
					// echo "Setting Base to: [".$val."]\n";
					break;
				case "continue":
					if ($val==1) $continue=true;
					else $continue=false;
					break;
				case "debug":
					if ($val==1) $debug=true;
					else $debug=false;
					break;
				case "default":
					$REX->Default=$val;
					break;
					
				case "set": case "env":
					if (!isset($argv[$a+1]))
						{
						echo "set or env need following variable=value set\n";
						CLI_Usage();
						exit();
						}
					$toset=$argv[++$a];
					$equalpos=strpos($toset,"=");
					if ($equalpos===false)
					{
						$toset_val=0;
					}
					else
					{
						$toset_val=Strip_Quotes(substr($toset,$equalpos+1));
						$toset=substr($toset,0,$equalpos);
					}
					$toset=Strip_Quotes($toset);
					
					echo "Setting ".$opt." [".$toset."] to [".$toset_val."]\n";
					
					if ($opt=="set") $REX->Set($toset,$toset_val);
					else if ($opt=="env") $REX->EnvLoad($toset,$toset_val);
					break;
					
					break;
				
				default:
					echo "Illegal or unknown option: ".$opt."\n";
					CLI_Usage();
					exit();
			}
					
		}
		else
		{
			$filenames[] = $opt;
		}
	}
	if (empty($filenames))
		{
		echo "No REX code filename(s) specified\n";
		CLI_Usage();
		exit();
		}
}
else if ($HTTP)
{ // rex_debug, rex_load_http, rex_continue, rex_quiet, rex_mute
  // rex_filename ? []
  // rex_set[] rex_val[]
  // rex_env_set[] rex_env_val[]
	if (isset($_REQUEST['rex_debug']) && ($_REQUEST['rex_debug']==1)) $debug=true;
	else $debug=false;
	
	if (isset($_REQUEST['rex_load_http']) && ($_REQUEST['rex_load_http']==1)) $REX->EnvLoadHTTP();
	
	if (isset($_REQUEST['rex_continue']) && ($_REQUEST['rex_continue']==1)) $continue=true;
	else $continue=false;
	
	if (isset($_REQUEST['rex_quiet']) && ($_REQUEST['rex_quiet']==1)) $REX->Quiet=true;
	else $REX->Quiet=false;
	
	if (isset($_REQUEST['rex_mute']) && ($_REQUEST['rex_mute']==1)) $mute=true;
	else $mute=false;
	
	if (isset($_REQUEST['rex_base']) && ($_REQUEST['rex_base']!="")) $REX->Base=$_REUEST['rex_base'];
	
	if (isset($_REQUEST['rex_filename']))
	{
		if (is_array($_REQUEST['rex_filename']))
		{
			foreach($_REQUEST['rex_filename'] as $filename) $filenames[]=$filename;
		}
		else $filenames[]=$_REQUEST['rex_filename'];
	}
	
	if ((isset($_REQUEST['rex_set'])) && (is_array($_REQUEST['rex_set'])))
	{
		foreach($_REQUEST['rex_set'] as $key => $val)
		{
			if ($val!="")
			{
				$variable = $val;
				if (isset($_REQUEST['rex_val'][$key])) $value=$_REQUEST['rex_val'][$key];
				else $value="";
				$REX->Set($variable, $value);
			}
		}
	}
	
if ((isset($_REQUEST['rex_env_set'])) && (is_array($_REQUEST['rex_env_set'])))
	{
		foreach($_REQUEST['rex_env_set'] as $key => $val)
		{
			if ($val!="")
			{
				$variable = $val;
				if (isset($_REQUEST['rex_env_val'][$key])) $value=$_REQUEST['rex_env_val'][$key];
				else $value="";
				$REX->EnvLoad($variable, $value);
			}
		}
	}
	
	
}

if ($debug)
{
	echo "REX Debug Environment Information:".Newline();
	echo " Initial Base : ".$REX->Base.Newline();
	echo " REX Quiet    : ".YesNo($REX->Quiet).Newline();
	echo " Blocks       : ".$REX->Blocks.Newline();
	echo " Default      : ".$REX->Default.Newline();
	echo "REX Exec Script Environment Information:".Newline();
	echo " Mute         : ".YesNo($mute).Newline();
	echo " Continue     : ".YesNo($continue).Newline();
	echo "REX Variables:".Newline();
	foreach($REX->Var as $key => $val)
		echo " ".$key." = ".$val.Newline();
	echo "REX Environment Variables:".Newline();
	foreach($REX->Env as $key => $val)
		echo " ".$key." = ".$val.Newline();
}

// Now the actual execution...
foreach($filenames as $filename)
{
	$REX->Error=false;
	$REX->ErrorList=array();
	
	if ($debug) echo "Executing: ".$filename.Newline().Newline();
	
	$result = $REX->Exec($filename);
	
	if ((!$mute) && (($result===false) || ($REX->Error==true)))
	{
		echo "REX Encountered an error executing: ".$filename.Newline();
		foreach($REX->ErrorList as $error)
		{
			echo "REX: ".$error.Newline();
		}
		if (!$continue)
		{
			echo "REX Exiting".Newline();
			exit();
		}	
	}
	
	if ($debug) echo Newline()."Finished Executing: ".$filename.Newline();
}


?>